"""
----------------------------------------------------------------------------------------------------
Written by:
  - Yovany Dominico Girón (y.dominico.giron@elprat.cat)

for Ajuntament del Prat de Llobregat
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
"""
from datetime import datetime, date, time
from typing import Any, Optional, Dict, Union, Literal, List, Tuple

from pydantic import BaseModel, RootModel, Field

from nomenclators_archetype.domain.repository.builders import COMPARISON_OPERATIONS, LITERAL_OPERATIONS


class SchemaPagination(BaseModel):
    """Schema Pagination class"""

    page: int = Field(
        ge=0, description="Número de página (en caso de utilizarse, debe ser >= 0)")
    element_for_page: int = Field(
        gt=0, description="Elementos por página (en caso de utilizarse, debe ser > 0)")
    limit: int = Field(
        gt=0, description="Máximo total de elementos a analizar (en caso de utilizarse, debe ser > 0)")
    element_sort: str = Field(...,
                              description="Campo por el cual se desea ordenar")
    sort: Literal["ASC", "DESC"] = Field(
        "DESC", description="Orden 'ASC' para ordenar Ascendentemente o 'DESC' para ordenar Descendentemente")

    def sort_mapped(self) -> List[Tuple[str, str]]:
        """Get sorte mapped"""
        return [(self.element_sort, self.sort)]


class SchemaFilterValue(BaseModel):
    """Schema FilterValue class: allows you to assign a direct value, a like/not like or a range"""

    eq: Optional[Union[str, int, float, datetime, date, time]] = None
    like: Optional[str] = None
    not_like: Optional[str] = None
    gt: Optional[Union[int, float, datetime, date, time]] = None
    lt: Optional[Union[int, float, datetime, date, time]] = None


class SchemaFilters(RootModel[Dict[str, SchemaFilterValue]]):
    """Schema Filters class: allows you to assign a filter to a field"""

    def filters_mapped(self) -> Dict[str, Any]:
        """Get filters mapped"""

        filters = {}
        for key, value in self.root.items():

            if isinstance(value, list):
                for item in list(value):
                    for sub_key, sub_value in item.items():
                        filters.update(self._map_filtre(
                            key, sub_key, sub_value))
            elif isinstance(value, SchemaFilterValue):
                for sub_key, sub_value in value.model_dump().items():
                    filters.update(self._map_filtre(
                        key, sub_key, sub_value))
            else:
                filters[key] = value

        return filters

    @staticmethod
    def _map_filtre(key: str, comparison_key: str, comparison_value: Any) -> dict:
        """
        Apply map filter to the key definition

        :param key: Key to filter
        :param comparison_key: comparing operator
        :param comparison_value: Value to comparison
        """
        if (
            (comparison_key.upper() in COMPARISON_OPERATIONS and isinstance(comparison_value, (str, int, float, datetime, date, time))) or
            (comparison_key.upper() in LITERAL_OPERATIONS and isinstance(
                comparison_value, str))
        ):
            return {f"{key}_{comparison_key.lower()}": comparison_value}
        else:
            return {}


class SchemaQueryParam(BaseModel):
    """Schema QueryParam class: allows you to assign a pagination and filter"""

    pagination: SchemaPagination
    filter: SchemaFilters
